/*
 * main.c
 */

#include "print.h"
#include "native.h"

extern int __text_start[];
extern int __text_end[];
extern int __rodata_start[];
extern int __rodata_end[];
extern int __data_start[];
extern int __data_end[];
extern int __bss_start[];
extern int __bss_end[];

static const char text[] = "HELLO WORLD! Hello World! hello world!";

static void relocate_native(void)
{
  unsigned int *dst = (unsigned int *)_native_jump_base;
  unsigned int x;

  // relocate
  for (x = (unsigned int)__native_start; x < (unsigned int)__native_end; x += 4)
  {
    *dst++ = *((unsigned int *)x);
  }
}

int main(void)
{
  unsigned int x, ypos, xpos;

  // clear bss
  for (x = (unsigned int)__bss_start; x < (unsigned int)__bss_end; x += 4)
  {
    *((unsigned int *)x) = 0;
  }

  relocate_native();

  _supercpu_turbo(1);

  print_clear();

  print_text(0, 0, text);

  print_hex(0, 2, 0x01234567);
  print_hex(0, 3, 0x89abcdef);

  print_dec(11, 2, 12345, 8, 0);
  print_dec(11, 3, 67890, 8, 1);

  print_text(0, 5, "ASCII");
  print_text(0, 5 + 10, "SCREEN");

  unsigned char *dst = (unsigned char *)1024;

  ypos = 6;
  xpos = 0;
  for (x = 0; x < 256; x++)
  {
    dst[((ypos + 10) * 40) + xpos] = x;
    print_char(xpos, ypos, x);
    xpos++;
    if (xpos == 32)
    {
      xpos = 0;
      ypos++;
    }
  }

  _tick_install();

  for (;;)
  {
    unsigned int t = _tick();
    print_text(20, 2, "TICKS: ");
    print_hex(27, 2, t);

    unsigned int seconds = (t / 100);
    unsigned int decimals = (t % 100);
    print_text(20, 3, "SECS : ");
    int x = print_dec(27, 3, seconds, 0, 0);
    x = print_char(x, 3, '.');
    x = print_dec(x, 3, decimals, 2, 1);
  }

  _tick_uninstall();

  return 0;
}
