/*
 * print.c
 */

#include "print.h"

/* shifted (lowercase) mode */
static const unsigned char ascii_to_screen[256] = {
  /* basic */
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f,
  0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f,
  0x00, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f,
  0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x1b, 0x5d, 0x1d, 0x71, 0x64,
  0x27, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
  0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x5d, 0x1d, 0x40, 0x20,
  /* extended */
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
};

void print_clear(void)
{
  unsigned char *dst = (unsigned char *)1024;
  int i;

  for (i = 0; i < (40 * 25); i++)
  {
    dst[i] = ascii_to_screen[' '];
  }

  /* switch to shifted (lowercase) mode */
  dst = (unsigned char *)0xd018;
  *dst |= 0x02;
}

int print_char(int x, int y, const char c)
{
  unsigned char *dst = (unsigned char *)1024;

  dst += (y * 40);
  dst += x;

  dst[0] = ascii_to_screen[(unsigned char)c];
  x++;

  return x;
}

int print_text(int x, int y, const char *text)
{
  unsigned char *dst = (unsigned char *)1024;
  unsigned char *src = (unsigned char *)text;

  dst += (y * 40);
  dst += x;

  while (*src != '\0')
  {
    dst[0] = ascii_to_screen[src[0]];
    dst++;
    src++;
    x++;
  }

  return x;
}

static const unsigned char hex_string[] = "0123456789ABCDEF";

int print_hex(int x, int y, unsigned int hex)
{
  print_char(x + 0, y, '0');
  print_char(x + 1, y, 'x');
  print_char(x + 2, y, hex_string[((hex >> 28) & 0x0f)]);
  print_char(x + 3, y, hex_string[((hex >> 24) & 0x0f)]);
  print_char(x + 4, y, hex_string[((hex >> 20) & 0x0f)]);
  print_char(x + 5, y, hex_string[((hex >> 16) & 0x0f)]);
  print_char(x + 6, y, hex_string[((hex >> 12) & 0x0f)]);
  print_char(x + 7, y, hex_string[((hex >>  8) & 0x0f)]);
  print_char(x + 8, y, hex_string[((hex >>  4) & 0x0f)]);
  print_char(x + 9, y, hex_string[((hex >>  0) & 0x0f)]);
  return x + 10;
}

int print_dec(int x, int y, unsigned int dec, int length, int zero_pad)
{
  char buffer[16];
  int i = 0;

  do
  {
    buffer[i] = '0' + (dec % 10);
    i++;
    dec /= 10;
  } while (dec > 0);

  while (i < length)
  {
    if (zero_pad)
    {
      buffer[i] = '0';
    }
    else
    {
      buffer[i] = ' ';
    }
    i++;
  }

  while (i > 0)
  {
    print_char(x, y, buffer[i - 1]);
    x++;
    i--;
  }

  return x;
}
